#!/usr/bin/env ruby

if ARGV.size < 3 || ARGV.first == "-h"
  puts "args: <out_prefix> <combo.bgl> <combo.ori> ALL.DAG..."
  puts "  * Make sure ori file matches same index order as bgl file"
  puts "  * Outputs <out_prefix>.<origin>.dag for all origins"
  exit
end

out_prefix=ARGV.shift

STDERR.puts("* Starting thread_dag with params [#{ARGV.join(" ")}]")
STDERR.puts("Reading Haplotype names from first line of #{ARGV.first}")
f = File.new(ARGV.shift).gets.chomp.split
hap_names = f[2, f.size]

hap_oris = {}
origin_ios = {}

STDERR.puts("Reading origins of those haplotypes")
origin_file = File.new(ARGV.shift)

for line in origin_file
  f = line.chomp.split

  # line is like: HGDP00001	51	Brahui	Pakistan
  # or it could be like line is like: HGDP00001A	23.123	34.34	Brahui	Pakistan
  hap_name = f[0]
  origin = f[3]

  if origin_ios[origin].nil?
    STDERR.puts "Origin found: #{origin}"
    origin_ios[origin] = File.new(out_prefix + "." + origin + ".dag", "w")
  end

  hap_oris[hap_name] = origin
end

STDERR.puts("Reading/Writing dags")
header = ARGF.gets.chomp
for io in origin_ios.values
  io.puts header
end

for line in ARGF
  # Just print any blank lines
  if line.strip.size == 0 
      for io in origin_ios.values
        io.puts line
      end
    next
  end

  f = line.split
  first_fields = f[0,5]
  count = f[5].to_i

  ori_indices = {} # origin  => [indices]
  for ori in hap_oris.values
    ori_indices[ori] = []
  end

  for x in f[6,f.size] do
    begin
      i = x.to_i
      name = hap_names[i]
      ori = hap_oris[name]
      ori_indices[ori] << i
    rescue
      STDERR.puts "Exception: hap_index=#{x} name=#{name} ori=#{hap_oris[name]} ori_indices[ori]=#{ori_indices[ori]}"
      exit 1
    end
  end

  for ori, indices in ori_indices
    raise unless indices.size == indices.uniq.size
    new_count = indices.size
    io = origin_ios[ori]
    io.print first_fields.join("\t")
    io.print "\t#{new_count}\t"
    io.puts indices.join("\t")
  end

end

for io in origin_ios.values
  io.close
end


STDERR.puts("* Done")
